<?php


namespace IPS\richdiscord\modules\admin\general;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * approvalqueue
 */
class _approvalqueue extends \IPS\Dispatcher\Controller
{	
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'approvalqueue_manage' );
		parent::execute();
	}
	
	/**
	 * Manage
	 *
	 * @return	void
	 */
	protected function manage()
	{
		if ( !\IPS\Application::load('richdiscord')->isConfigured() ) {
		    \IPS\Output::i()->redirect(\IPS\Http\Url::internal('app=richdiscord&module=general&controller=configuration'));
        }
		if ( !isset(\IPS\Data\Store::i()->richdiscord_queue) ) {
		    \IPS\Task::constructFromData( \IPS\Db::i()->select('*', 'core_tasks', array('app=? AND `key`=?', 'richdiscord', 'refreshwaitingusers') )->first())->runAndLog();
        }
        $content = array();
		foreach (\IPS\Data\Store::i()->richdiscord_queue as $k => $v) {
		    $content[] = [ 'member_id' => $k, 'joined_at' => $v['joined_at'] ];
        }
		$table = new \IPS\Helpers\Table\Custom( $content, \IPS\Http\Url::internal( 'app=richdiscord&module=general&controller=approvalqueue' ) );
		$table->langPrefix = 'richdiscord_queue_';
		$table->quickSearch = function( $row )
        {
            return mb_strpos(mb_strtolower(\IPS\Member::load($row['member_id'])->name), mb_strtolower(\IPS\Request::i()->quicksearch)) !== FALSE;
        };

        $table->rowButtons = function( $row )
        {
            $buttons = [];
            $rid = $row['member_id'];
            $buttons[ 'approve' ] = [
                'icon'		=> 'check',
                'title'		=> 'richdiscord_queue_approve',
                'id'		=> "$rid-approve",
                'link'		=> \IPS\Http\Url::internal( 'app=richdiscord&module=general&controller=approvalqueue&do=approve' )->setQueryString( array( 'id' => $rid ) )
            ];

            $buttons[ 'kick' ] = [
                'icon'		=> 'arrow-left',
                'title'		=> 'richdiscord_queue_kick',
                'id'		=> "$rid-kick",
                'link'		=> \IPS\Http\Url::internal( 'app=richdiscord&module=general&controller=approvalqueue&do=kick' )->setQueryString( array( 'id' => $rid ) )
            ];

            $buttons[ 'ban' ] = [
                'icon'		=> 'lock',
                'title'		=> 'richdiscord_queue_ban',
                'id'		=> "$rid-ban",
                'link'		=> \IPS\Http\Url::internal( 'app=richdiscord&module=general&controller=approvalqueue&do=ban' )->setQueryString( array( 'id' => $rid ) )
            ];
            return $buttons;
        };

        $table->rootButtons = [
            'refresh' => [
                'icon'		=> 'refresh',
                'title'		=> 'richdiscord_queue_refresh',
                'id'		=> "queue-refresh",
                'link'		=> \IPS\Http\Url::internal( 'app=richdiscord&module=general&controller=approvalqueue&do=refresh' ),
                'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'richdiscord_queue_refresh' ) ),
            ]
        ];

		$table->parsers = [
		    'joined_at' => function ($val) {
		        return \IPS\DateTime::ts(date("U",strtotime($val)))->html();
            },
            'member_id' => function ($val) {
                try
                {
                    $member = \IPS\Member::load( $val );
                    return \IPS\Theme::i()->getTemplate( 'global', 'core' )->userPhoto( $member, 'tiny' ) . " <a style='margin-left:10px' href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $member->member_id . "'>" . htmlentities( $member->name, \IPS\HTMLENTITIES, 'UTF-8', FALSE ) . "</a>";
                }
                catch( \OutOfRangeException $e )
                {
                    return $val;
                }
            }
        ];
		/* Display */
		\IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'title', "<div class='ipsPad'>$table</div>" );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('menu__richdiscord_general_queue');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'title', "<div class='ipsPad'>$table</div>" );
	}

	public function approve()
    {
	    if (!\IPS\Request::i()->id) {
	        \IPS\Output::i()->error("richdiscord_no_id", "RD/N01D", 400);
        }
        try {
	        $member = \IPS\Member::load(\IPS\Request::i()->id);
        } catch (\OutOfRangeException $e) {
            \IPS\Output::i()->error("richdiscord_bad_id", "RD/B4D1D", 400);
        }
        $member->richdiscordSync(TRUE);
	    $queue = \IPS\Data\Store::i()->richdiscord_queue;
	    unset($queue[\IPS\Request::i()->id]);
	    \IPS\Data\Store::i()->richdiscord_queue = $queue;
        \IPS\Output::i()->redirect(\IPS\Http\Url::internal("app=richdiscord&module=general&controller=approvalqueue"), "saved");
    }

    public function kick()
    {
        if (!\IPS\Request::i()->id) {
            \IPS\Output::i()->error("richdiscord_no_id", "RD/N01D", 400);
        }
        try {
            $member = \IPS\Member::load(\IPS\Request::i()->id);
        } catch (\OutOfRangeException $e) {
            \IPS\Output::i()->error("richdiscord_bad_id", "RD/B4D1D", 400);
        }
        $member->richdiscordKick();
        $queue = \IPS\Data\Store::i()->richdiscord_queue;
        unset($queue[\IPS\Request::i()->id]);
        \IPS\Data\Store::i()->richdiscord_queue = $queue;
        \IPS\Output::i()->redirect(\IPS\Http\Url::internal("app=richdiscord&module=general&controller=approvalqueue"), "saved");
    }

    public function ban()
    {
        if (!\IPS\Request::i()->id) {
            \IPS\Output::i()->error("richdiscord_no_id", "RD/N01D", 400);
        }
        try {
            $member = \IPS\Member::load(\IPS\Request::i()->id);
        } catch (\OutOfRangeException $e) {
            \IPS\Output::i()->error("richdiscord_bad_id", "RD/B4D1D", 400);
        }
        $member->richdiscordBan();
        $queue = \IPS\Data\Store::i()->richdiscord_queue;
        unset($queue[\IPS\Request::i()->id]);
        \IPS\Data\Store::i()->richdiscord_queue = $queue;
        \IPS\Output::i()->redirect(\IPS\Http\Url::internal("app=richdiscord&module=general&controller=approvalqueue"), "saved");
    }

	public function refresh( $redirect=TRUE )
    {
        \IPS\Task::constructFromData( \IPS\Db::i()->select('*', 'core_tasks', array('app=? AND `key`=?', 'richdiscord', 'refreshwaitingusers') )->first())->runAndLog();
        if ( $redirect ) {
            \IPS\Output::i()->redirect(\IPS\Http\Url::internal( 'app=richdiscord&module=general&controller=approvalqueue' ) );
        }
    }
}