<?php

namespace IPS\richdiscord\Discord;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
    header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
    exit;
}

class _Guild extends Requestable
{
    public $id;
    public $name;
    public $iconHash;
    public $splashHash;
    public $ownerId;
    public $permissions;
    public $voiceRegion;
    public $afkChannelId;
    public $afkTimeout;
    public $embedEnabled;
    public $embedChannelId;
    public $verificationLevel;
    public $notificationsSettings;
    public $roles;
    public $emojis;
    public $mfaLevel;
    public $systemChannelId;
    public $members;

    public static function make( $id, $resolveMembers=FALSE )
    {
        try {
            $data = parent::request("/guilds/$id")->decodeJson();
        } catch (Exceptions\ForbiddenException $e) {
            return NULL;
        }
        $guild = new Guild();
        $guild->id = $data['id'];
        $guild->name = $data['name'];
        $guild->iconHash = $data['icon'];
        $guild->splashHash = $data['splash'];
        $guild->ownerId = $data['owner_id'];
        $guild->permissions = @$data['permissions'];
        $guild->voiceRegion = $data['region'];
        $guild->afkChannelId = $data['afk_channel_id'];
        $guild->afkTimeout = $data['afk_timeout'];
        $guild->embedEnabled = $data['embed_enabled'];
        $guild->embedChannelId = $data['embed_channel_id'];
        $guild->verificationLevel = $data['verification_level'];
        $guild->notificationsSettings = $data['default_message_notifications'];
        $guild->emojis = array_map(function ($val) {
            $v = new Guild\Emoji;
            $v->id = $val['id'];
            $v->name = $val['name'];
            $v->roleIds = $val['roles'];
            $v->requireColons = $val['require_colons'];
            return $v;
        }, $data['emojis']);
        $guild->roles = array_map(function ($val) {
            $v = new Guild\Role;
            $v->id = $val['id'];
            $v->hoist = $val['hoist'];
            $v->mentionable = $val['mentionable'];
            $v->color = $val['color'];
            $v->name = $val['name'];
            $v->position = $val['position'];
            $v->managed = $val['managed'];
            $v->permissions = $val['permissions'];
            return $v;
        }, $data['roles']);
        $guild->mfaLevel = $data['mfa_level'];
        $guild->systemChannelId = $data['system_channel_id'];
        if ($resolveMembers) {
            $guild->members = $guild->members();
        }
        return $guild;
    }

    /**
     * Get members.
     * @param int $perpage
     * @param int $after
     * @return array
     */
    public function members( $perpage=1000, $after=0 )
    {
        $result = [];
        while (true) {
            $json = parent::request("/guilds/{$this->id}/members?limit=$perpage&after=$after")->decodeJson();
            $result = array_merge($result, $json);
            if (count($json) != $perpage ) {
                break;
            } else {
                $after = max(array_map(function($val){return $val['user']['id'];}, $json));
            }
        }
        return $result;
    }
}