<?php

namespace IPS\richdiscord\Discord;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
    header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
    exit;
}

abstract class _Requestable
{
    const API_URL = 'https://discordapp.com/api/v6';
    const OAUTH2_URL = 'https://discordapp.com/api/v6/oauth2';

    public static function request( $path, $method='GET', $body='', $bodyContentType='application/json', $token=NULL, $customHeaders=array(), $rateLimitRetry=FALSE )
    {
        $method = mb_strtolower( $method );
        $request = \IPS\Http\Url::external( static::API_URL . $path )
            ->request()
            ->setHeaders( array_merge([
                'Authorization' => 'Bot ' . ( ($token) ? $token : \IPS\Settings::i()->richdiscord_token ),
                'User-Agent' => 'DiscordBot (ipbrichdiscordintegration, v1)',
                'Content-Type' => $bodyContentType
            ], $customHeaders) );
        if ( $method == 'get' )
        {
            $response = $request->get();
        }
        else
        {
            $response =  $request->$method( $body );
        }

        if ( !in_array( $response->httpResponseCode, [200, 201, 204, 304] ) )
        {
            switch ( (string) $response->httpResponseCode ) {
                case "400":
                    throw new Exceptions\BadRequestException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                case "403":
                    throw new Exceptions\ForbiddenException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                case "502":
                    throw new Exceptions\GatewayUnavailableException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                case "405":
                    throw new Exceptions\MethodNotAllowedException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                case "404":
                    throw new Exceptions\NotFoundException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                case "429":
                    if ($rateLimitRetry) {
                        sleep($response->httpResponseHeaders['X-RateLimit-Reset'] - time());
                        return static::request($path, $method, $body, $bodyContentType, $token, $customHeaders, $rateLimitRetry);
                    }
                    throw new Exceptions\TooManyRequestsException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                case "401":
                    throw new Exceptions\UnauthorizedException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                default:
                    if (mb_substr((string)$response->httpResponseCode, 0, 1) == "5") {
                        throw new Exceptions\ServerErrorException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
                    }
                    throw new Exceptions\UnknownErrorException("Error on endpoint $path: ".((@$response->decodeJson()['error']) ? $response->decodeJson()['error'] : $response->httpResponseText));
            }
        }

        return $response;
    }
}