<?php
/**
 * @brief		refreshwaitingusers Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	richdiscord
 * @since		10 Mar 2018
 */

namespace IPS\richdiscord\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * refreshwaitingusers Task
 */
class _refreshwaitingusers extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
	    if ( !\IPS\Application::load( 'richdiscord' )->isConfigured() ) return NULL;
	    $this->_checkUsers();
		return NULL;
	}

	protected function _checkUsers()
    {
        if (!\IPS\Application::load('richdiscord')->isConfigured()) {
            return NULL;
        }
        $managed = [];
        foreach (\IPS\Member\Group::groups(TRUE, FALSE) as $v) {
            $managed = array_merge($managed, array_filter($v->g_richdiscord_roles, 'strlen'));
        }
        $managed = array_unique($managed);
        $guild = null;
        try {
            $guild = \IPS\richdiscord\Discord\Guild::make( \IPS\Settings::i()->richdiscord_guild, TRUE );
        } catch ( \Exception $e ) {
            \IPS\Log::log( $e, 'discord_error' );
            throw new \IPS\Task\Exception( $this, 'richdiscord_failed_to_load_guild' );
        }
        $queue = [];
        $members = array_filter($guild->members, function($val){
            if (@$val['user']['bot'] == TRUE)
                return FALSE;
            else return TRUE;
        });
        // https://stackoverflow.com/questions/3145607/php-check-if-an-array-has-duplicates/43635394#43635394
        $array_has_dupes = function($array) {
            return count($array) != count(array_flip($array));
        };
        foreach ($members as $v) {
            if (!$array_has_dupes(array_merge($managed, $v['roles']))) {
                try {
                    $member = \IPS\Db::i()->select( 'member_id, mgroup_others, member_group_id', 'core_members', array('richdiscord_id=? AND temp_ban=0', $v['user']['id']) )->first();
                    $roles = iterator_to_array(\IPS\Db::i()->select( 'g_richdiscord_roles', 'core_groups', array( \IPS\Db::i()->in('g_id', array_merge(array_filter(explode(',', $member['mgroup_others']), 'strlen'), [$member['member_group_id']]) ) ) ));
                    $rroles = [];
                    $roles = array_filter($roles, function ($vv) {return !is_null($vv);});
                    foreach ($roles as $role) {
                        $rroles = array_merge($rroles, array_filter(explode(',', $role), 'strlen'));
                    }
                    if (count($rroles)) {
                        $queue[$member['member_id']] = $v;
                    }
                } catch ( \UnderflowException $e ) {}
            }
        }
        \IPS\Data\Store::i()->richdiscord_queue = $queue;
    }
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}