<?php
/**
 * @brief		syncdiscordusers Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	richdiscord
 * @since		28 Apr 2018
 */

namespace IPS\richdiscord\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * syncdiscordusers Task
 */
class _syncdiscordusers extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
	    if (!\IPS\Application::load('richdiscord')->isConfigured()) {
	        return NULL;
        }
        $managed = [];
        foreach (\IPS\Member\Group::groups(TRUE, FALSE) as $v) {
            $managed = array_merge($managed, is_array($v->g_richdiscord_roles) ? $v->g_richdiscord_roles : [$v->g_richdiscord_roles]);
        }
        $managed = array_unique($managed);
        $guild = null;
        try {
            $guild = \IPS\richdiscord\Discord\Guild::make( \IPS\Settings::i()->richdiscord_guild, TRUE );
        } catch ( \Exception $e ) {
            \IPS\Log::log( $e, 'discord_error' );
            throw new \IPS\Task\Exception( $this, 'richdiscord_failed_to_load_guild' );
        }
        $members = array_filter($guild->members, function($val){
            if (@$val['user']['bot'] == TRUE)
                return FALSE;
            else return TRUE;
        });
        foreach ($members as $member) {
            try {
                //throw new \OutOfRangeException(json_encode($member['user']['id']));
                $member = \IPS\Db::i()->select( '*', 'core_members', array('temp_ban=0 AND richdiscord_id=?', $member['user']['id']) )->first();
                $member = \IPS\Member::constructFromData($member);
                try {
                    $member->richdiscordSync( FALSE, $managed );
                } catch ( \IPS\richdiscord\Discord\Exceptions\BaseException $e ) {
                    \IPS\Log::log( $e, 'discord_error' );
                }
            } catch ( \UnderflowException $e ) {}
        }
		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}